@@ -16,10 +16,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, 
+ * Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA  02110-1301  USA
  */
- 
+
 #include <config.h>
 
 #include <gtk/gtk.h>
@@ -35,14 +35,9 @@
 #include "callbacks.h"
 #include "baobab-prefs.h"
 
-#include "gedit-spinner.h"
-
 #include "baobab-treemap.h"
 #include "baobab-ringschart.h"
 
-static GVolumeMonitor 	 *monitor_vol;
-static GFileMonitor	 *monitor_home;
-
 static void push_iter_in_stack (GtkTreeIter *);
 static GtkTreeIter pop_iter_from_stack (void);
 
@@ -51,11 +46,6 @@ static GtkTreeIter currentiter;
 static GtkTreeIter firstiter;
 static GQueue *iterstack = NULL;
 
-static GFile *current_location = NULL;
-
-#define BUSY_IMAGE_PATH		BAOBAB_PIX_DIR "busy.gif"
-#define DONE_IMAGE_PATH		BAOBAB_PIX_DIR "done.png"
-
 enum {
 	DND_TARGET_URI_LIST
 };
@@ -84,6 +74,7 @@ baobab_set_busy (gboolean busy)
 {
 	static GdkCursor *busy_cursor = NULL;
 	GdkCursor *cursor = NULL;
+	GdkWindow *window;
 
 	if (busy == TRUE) {
 		if (!busy_cursor) {
@@ -91,7 +82,8 @@ baobab_set_busy (gboolean busy)
 		}
 		cursor = busy_cursor;
 
-		gedit_spinner_start (GEDIT_SPINNER (baobab.spinner));
+		gtk_widget_show (baobab.spinner);
+		gtk_spinner_start (GTK_SPINNER (baobab.spinner));
 
 		baobab_chart_freeze_updates (baobab.rings_chart);
 
@@ -100,7 +92,8 @@ baobab_set_busy (gboolean busy)
 		gtk_widget_set_sensitive (baobab.chart_type_combo, FALSE);
 	}
 	else {
-		gedit_spinner_stop (GEDIT_SPINNER (baobab.spinner));
+		gtk_widget_hide (baobab.spinner);
+		gtk_spinner_stop (GTK_SPINNER (baobab.spinner));
 
 		baobab_chart_thaw_updates (baobab.rings_chart);
 		baobab_chart_thaw_updates (baobab.treemap_chart);
@@ -109,8 +102,9 @@ baobab_set_busy (gboolean busy)
 	}
 
 	/* change the cursor */
-	if (baobab.window->window) {
-		gdk_window_set_cursor (baobab.window->window, cursor);
+	window = gtk_widget_get_window (baobab.window);
+	if (window) {
+		gdk_window_set_cursor (window, cursor);
 	}
 }
 
@@ -131,32 +125,33 @@ set_drop_target (GtkWidget *target, gboolean active) {
 static void
 check_menu_sens (gboolean scanning)
 {
-	if (scanning) {
+	gboolean has_current_location;
 
+	if (scanning) {
 		while (gtk_events_pending ())
 			gtk_main_iteration ();
 
-		set_statusbar (_("Scanning..."));
+		baobab_set_statusbar (_("Scanning..."));
 		set_ui_action_sens ("expand_all", TRUE);
 		set_ui_action_sens ("collapse_all", TRUE);
 	}
 
+	has_current_location = baobab.current_location != NULL;
+
 	set_ui_action_sens ("menuscanhome", !scanning);
 	set_ui_action_sens ("menuallfs", !scanning);
 	set_ui_action_sens ("menuscandir", !scanning);
 	set_ui_action_sens ("menustop", scanning);
-	set_ui_action_sens ("menurescan", !scanning && current_location != NULL);
+	set_ui_action_sens ("menurescan", !scanning && has_current_location);
 	set_ui_action_sens ("preferenze1", !scanning);
 	set_ui_action_sens ("menu_scan_rem", !scanning);
-	set_ui_action_sens ("ck_allocated",
-			    !scanning &&
-			    baobab.is_local && !g_noactivescans);
+	set_ui_action_sens ("ck_allocated", !scanning && baobab.is_local);
 
 	set_ui_widget_sens ("tbscanhome", !scanning);
 	set_ui_widget_sens ("tbscanall", !scanning);
 	set_ui_widget_sens ("tbscandir", !scanning);
 	set_ui_widget_sens ("tbstop", scanning);
-	set_ui_widget_sens ("tbrescan", !scanning && current_location != NULL);
+	set_ui_widget_sens ("tbrescan", !scanning && has_current_location);
 	set_ui_widget_sens ("tb_scan_remote", !scanning);
 }
 
@@ -167,6 +162,41 @@ check_drop_targets (gboolean scanning)
 	set_drop_target (baobab.treemap_chart, !scanning);
 }
 
+static void
+update_scan_label (void)
+{
+	gchar *markup;
+	gchar *total;
+	gchar *used;
+	gchar *available;
+	GtkWidget *label;
+
+	total = g_format_size_for_display (baobab.fs.total);
+	used = g_format_size_for_display (baobab.fs.used);
+	available = g_format_size_for_display (baobab.fs.avail);
+
+	/* Translators: these are labels for disk space */
+	markup = g_markup_printf_escaped  ("<small>%s <b>%s</b> (%s %s %s %s )</small>",
+					   _("Total filesystem capacity:"), total,
+					   _("used:"), used,
+					   _("available:"), available);
+
+	g_free (total);
+	g_free (used);
+	g_free (available);
+
+	label = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "label1"));
+
+	gtk_label_set_markup (GTK_LABEL (label), markup);
+}
+
+void
+baobab_update_filesystem (void)
+{
+	baobab_get_filesystem (&baobab.fs);
+	update_scan_label ();
+}
+
 void
 baobab_scan_location (GFile *file)
 {
@@ -178,11 +208,10 @@ baobab_scan_location (GFile *file)
 	if (iterstack !=NULL)
 		return;
 
-	if (current_location)
-		g_object_unref (current_location);
-	current_location = g_object_ref (file);
+	if (baobab.current_location)
+		g_object_unref (baobab.current_location);
+	baobab.current_location = g_object_ref (file);
 
-	g_noactivescans = FALSE; 
 	baobab.STOP_SCANNING = FALSE;
 	baobab_set_busy (TRUE);
 	check_menu_sens (TRUE);
@@ -206,7 +235,7 @@ baobab_scan_location (GFile *file)
 	baobab_scan_execute (file);
 
 	/* set statusbar, percentage and allocated/normal size */
-	set_statusbar (_("Calculating percentage bars..."));
+	baobab_set_statusbar (_("Calculating percentage bars..."));
 	gtk_tree_model_foreach (GTK_TREE_MODEL (baobab.model),
 				show_bars,
 				NULL);
@@ -217,7 +246,7 @@ baobab_scan_location (GFile *file)
 	baobab_set_busy (FALSE);
 	check_menu_sens (FALSE);
 	check_drop_targets (FALSE);
-	set_statusbar (_("Ready"));
+	baobab_set_statusbar (_("Ready"));
 
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (baobab.tree_view));
 	baobab.STOP_SCANNING = TRUE;
@@ -249,15 +278,13 @@ baobab_scan_root (void)
 void
 baobab_rescan_current_dir (void)
 {
-	g_return_if_fail (current_location != NULL);
+	g_return_if_fail (baobab.current_location != NULL);
 
-	baobab_get_filesystem (&g_fs);
-	set_label_scan (&g_fs);
-	show_label ();
+	baobab_update_filesystem ();
 
-	g_object_ref (current_location);
-	baobab_scan_location (current_location);
-	g_object_unref (current_location);
+	g_object_ref (baobab.current_location);
+	baobab_scan_location (baobab.current_location);
+	g_object_unref (baobab.current_location);
 }
 
 void
@@ -265,7 +292,7 @@ baobab_stop_scan (void)
 {
 	baobab.STOP_SCANNING = TRUE;
 
-	set_statusbar (_("Calculating percentage bars..."));
+	baobab_set_statusbar (_("Calculating percentage bars..."));
 	gtk_tree_model_foreach (GTK_TREE_MODEL (baobab.model),
 				show_bars, NULL);
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (baobab.tree_view));
@@ -329,8 +356,8 @@ prefill_model (struct chan_data *data)
 	gtk_tree_store_set (baobab.model, &iter,
 			    COL_DIR_NAME, name,
 			    COL_H_PARSENAME, "",
-			    COL_H_ELEMENTS, -1, 
- 			    COL_H_PERC, -1.0,
+			    COL_H_ELEMENTS, -1,
+			    COL_H_PERC, -1.0,
 			    COL_DIR_SIZE, str,
 			    COL_ELEMENTS, str, -1);
 
@@ -342,22 +369,19 @@ prefill_model (struct chan_data *data)
 	}
 }
 
-/*
- * set filesystem first row
- */
-void
+static void
 first_row (void)
 {
 	char *size;
 	gdouble perc;
 	char *label;
 
-        GtkTreeIter root_iter;
+	GtkTreeIter root_iter;
 
 	gchar *capacity_label, *capacity_size;
 
 	gtk_tree_store_append (baobab.model, &root_iter, NULL);
-	capacity_size = g_format_size_for_display (g_fs.total);
+	capacity_size = g_format_size_for_display (baobab.fs.total);
 
 	capacity_label = g_strdup (_("Total filesystem capacity"));
 	gtk_tree_store_set (baobab.model, &root_iter,
@@ -365,20 +389,20 @@ first_row (void)
 			    COL_H_PARSENAME, "",
 			    COL_H_PERC, 100.0,
 			    COL_DIR_SIZE, capacity_size,
-			    COL_H_SIZE, g_fs.total,
-			    COL_H_ALLOCSIZE, g_fs.total,
+			    COL_H_SIZE, baobab.fs.total,
+			    COL_H_ALLOCSIZE, baobab.fs.total,
 			    COL_H_ELEMENTS, -1, -1);
 	g_free (capacity_label);
 	g_free (capacity_size);
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (baobab.tree_view), FALSE);
 	gtk_tree_store_append (baobab.model, &firstiter, &root_iter);
-	size = g_format_size_for_display (g_fs.used);
-	if (g_fs.total == 0 && g_fs.used == 0) {
+	size = g_format_size_for_display (baobab.fs.used);
+	if (baobab.fs.total == 0 && baobab.fs.used == 0) {
 		perc = 100.0;
 	} else {
-		g_assert (g_fs.total != 0);
-		perc = ((gdouble) g_fs.used * 100) / (gdouble) g_fs.total;
+		g_assert (baobab.fs.total != 0);
+		perc = ((gdouble) baobab.fs.used * 100) / (gdouble) baobab.fs.total;
 	}
 
 	label = g_strdup (_("Total filesystem usage"));
@@ -387,8 +411,8 @@ first_row (void)
 			    COL_H_PARSENAME, "",
 			    COL_H_PERC, perc,
 			    COL_DIR_SIZE, size,
-			    COL_H_SIZE, g_fs.used,
-			    COL_H_ALLOCSIZE, g_fs.used,
+			    COL_H_SIZE, baobab.fs.used,
+			    COL_H_ALLOCSIZE, baobab.fs.used,
 			    COL_H_ELEMENTS, -1, -1);
 
 	g_free (size);
@@ -399,7 +423,7 @@ first_row (void)
 
 /* fills model during scanning */
 void
-fill_model (struct chan_data *data)
+baobab_fill_model (struct chan_data *data)
 {
 	GtkTreeIter iter;
 	GString *hardlinks;
@@ -431,7 +455,7 @@ fill_model (struct chan_data *data)
 
 	elements = g_string_new ("");
 	g_string_printf (elements,
-			 ngettext ("% 5d item", "% 5d items",
+			 ngettext ("%5d item", "%5d items",
 				   data->elements), data->elements);
 
 	size = g_format_size_for_display (data->size);
@@ -483,20 +507,6 @@ pop_iter_from_stack (void)
 	return iter;
 }
 
-void
-baobab_set_excluded_locations (GSList *excluded_uris)
-{
-	GSList *l;
-
-	g_slist_foreach (baobab.excluded_locations, (GFunc) g_object_unref, NULL);
-	g_slist_free (baobab.excluded_locations);
-	baobab.excluded_locations = NULL;
-	for (l = excluded_uris; l != NULL; l = l->next) {
-		baobab.excluded_locations = g_slist_prepend (baobab.excluded_locations,
-						g_file_new_for_uri (l->data));
-	}
-}
-
 gboolean
 baobab_is_excluded_location (GFile *file)
 {
@@ -505,7 +515,7 @@ baobab_is_excluded_location (GFile *file)
 
 	g_return_val_if_fail (file != NULL, FALSE);
 
-	for (l = baobab.excluded_locations; l != NULL; l = l->next) {	
+	for (l = baobab.excluded_locations; l != NULL; l = l->next) {
 		if (g_file_equal (l->data, file)) {
 			ret = TRUE;
 			break;
@@ -515,8 +525,85 @@ baobab_is_excluded_location (GFile *file)
 	return ret;
 }
 
+static void
+volume_changed (GVolumeMonitor *volume_monitor,
+                GVolume        *volume,
+                gpointer        user_data)
+{
+	/* filesystem has changed (mounted or unmounted device) */
+	baobab_update_filesystem ();
+}
+
+static void
+home_contents_changed (GFileMonitor      *file_monitor,
+		       GFile             *child,
+		       GFile             *other_file,
+		       GFileMonitorEvent  event_type,
+		       gpointer           user_data)
+
+{
+	gchar *excluding;
+
+	if (baobab.CONTENTS_CHANGED_DELAYED)
+		return;
+
+	excluding = g_file_get_basename (child);
+	if (strcmp (excluding, ".recently-used") == 0   ||
+	    strcmp (excluding, ".gnome2_private") == 0  ||
+	    strcmp (excluding, ".xsession-errors") == 0 ||
+	    strcmp (excluding, ".bash_history") == 0    ||
+	    strcmp (excluding, ".gconfd") == 0) {
+		g_free (excluding);
+		return;
+	}
+	g_free (excluding);
+
+	baobab.CONTENTS_CHANGED_DELAYED = TRUE;
+}
+
+static void
+monitor_volume (void)
+{
+	baobab.monitor_vol = g_volume_monitor_get ();
+
+	g_signal_connect (baobab.monitor_vol, "volume_changed",
+			  G_CALLBACK (volume_changed), NULL);
+}
+
+static void
+monitor_home (gboolean enable)
+{
+	if (enable && baobab.monitor_home == NULL) {
+		GFile *file;
+		GError *error = NULL;
+
+		file = g_file_new_for_path (g_get_home_dir ());
+		baobab.monitor_home = g_file_monitor_directory (file, 0, NULL, &error);
+		g_object_unref (file);
+
+		if (!baobab.monitor_home) {
+			message (_("Could not initialize monitoring"),
+				 _("Changes to your home folder will not be monitored."),
+				 GTK_MESSAGE_WARNING, NULL);
+			g_print ("homedir:%s\n", error->message);
+			g_error_free (error);
+		}
+		else {
+			g_signal_connect (baobab.monitor_home,
+					  "changed",
+					  G_CALLBACK (home_contents_changed),
+					  NULL);
+		}
+	}
+	else if (!enable && baobab.monitor_home != NULL) {
+		g_file_monitor_cancel (baobab.monitor_home);
+		g_object_unref (baobab.monitor_home);
+		baobab.monitor_home = NULL;
+	}
+}
+
 void
-set_toolbar_visible (gboolean visible)
+baobab_set_toolbar_visible (gboolean visible)
 {
 	GtkToggleAction *action;
 
@@ -531,7 +618,7 @@ set_toolbar_visible (gboolean visible)
 }
 
 void
-set_statusbar_visible (gboolean visible)
+baobab_set_statusbar_visible (gboolean visible)
 {
 	GtkToggleAction *action;
 
@@ -546,7 +633,7 @@ set_statusbar_visible (gboolean visible)
 }
 
 void
-set_statusbar (const gchar *text)
+baobab_set_statusbar (const gchar *text)
 {
 	gtk_statusbar_pop (GTK_STATUSBAR (baobab.statusbar), 1);
 	gtk_statusbar_push (GTK_STATUSBAR (baobab.statusbar), 1, text);
@@ -556,8 +643,8 @@ set_statusbar (const gchar *text)
 }
 
 static void
-toolbar_reconfigured_cb (GtkToolItem  *item,
-			 GeditSpinner *spinner)
+toolbar_reconfigured_cb (GtkToolItem *item,
+			 GtkWidget   *spinner)
 {
 	GtkToolbarStyle style;
 	GtkIconSize size;
@@ -573,7 +660,7 @@ toolbar_reconfigured_cb (GtkToolItem  *item,
 		size = GTK_ICON_SIZE_LARGE_TOOLBAR;
 	}
 
-	gedit_spinner_set_size (spinner, size);
+	gtk_widget_set_size_request (spinner, size, size);
 }
 
 static void
@@ -585,7 +672,7 @@ baobab_toolbar_style (GConfClient *client,
 	gchar *toolbar_setting;
 
 	toolbar_setting = baobab_gconf_get_string_with_default (baobab.gconf_client,
-								SYSTEM_TOOLBAR_STYLE,
+								SYSTEM_TOOLBAR_STYLE_KEY,
 								"both");
 
 	if (!strcmp(toolbar_setting, "icons")) {
@@ -630,23 +717,23 @@ baobab_create_toolbar (void)
 	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (separator));
 	gtk_widget_show (GTK_WIDGET (separator));
 
-	baobab.spinner = gedit_spinner_new ();
+	baobab.spinner = gtk_spinner_new ();
 	item = gtk_tool_item_new ();
 	gtk_container_add (GTK_CONTAINER (item), baobab.spinner);
 	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (item));
-	gtk_widget_show (GTK_WIDGET (baobab.spinner));
 	gtk_widget_show (GTK_WIDGET (item));
 
 	g_signal_connect (item, "toolbar-reconfigured",
 			  G_CALLBACK (toolbar_reconfigured_cb), baobab.spinner);
-	toolbar_reconfigured_cb (item, GEDIT_SPINNER (baobab.spinner));
+	toolbar_reconfigured_cb (item, baobab.spinner);
+
 	baobab_toolbar_style (NULL, 0, NULL, NULL);
 
 	visible = gconf_client_get_bool (baobab.gconf_client,
 					 BAOBAB_TOOLBAR_VISIBLE_KEY,
 					 NULL);
 
-	set_toolbar_visible (visible);
+	baobab_set_toolbar_visible (visible);
 }
 
 static void
@@ -664,7 +751,8 @@ baobab_create_statusbar (void)
 	visible = gconf_client_get_bool (baobab.gconf_client,
 					 BAOBAB_STATUSBAR_VISIBLE_KEY,
 					 NULL);
-	set_statusbar_visible (visible);
+
+	baobab_set_statusbar_visible (visible);
 }
 
 static void
@@ -679,6 +767,20 @@ baobab_subfolderstips_toggled (GConfClient *client,
 									    NULL));
 }
 
+static void
+baobab_set_excluded_locations (GSList *excluded_uris)
+{
+	GSList *l;
+
+	g_slist_foreach (baobab.excluded_locations, (GFunc) g_object_unref, NULL);
+	g_slist_free (baobab.excluded_locations);
+	baobab.excluded_locations = NULL;
+	for (l = excluded_uris; l != NULL; l = l->next) {
+		baobab.excluded_locations = g_slist_prepend (baobab.excluded_locations,
+						g_file_new_for_uri (l->data));
+	}
+}
+
 static void
 store_excluded_locations (void)
 {
@@ -692,8 +794,8 @@ store_excluded_locations (void)
 	}
 
 	gconf_client_set_list (baobab.gconf_client,
-			       PROPS_SCAN_KEY,
-			       GCONF_VALUE_STRING, 
+			       BAOBAB_EXCLUDED_DIRS_KEY,
+			       GCONF_VALUE_STRING,
 			       uri_list,
 			       NULL);
 
@@ -714,7 +816,7 @@ sanity_check_excluded_locations (void)
 		if (g_file_equal (l->data, root)) {
 			baobab.excluded_locations = g_slist_delete_link (baobab.excluded_locations, l);
 			store_excluded_locations ();
-			break;			
+			break;
 		}
 	}
 
@@ -722,78 +824,40 @@ sanity_check_excluded_locations (void)
 }
 
 static void
-volume_changed (GVolumeMonitor *volume_monitor,
-                GVolume        *volume,
-                gpointer        user_data)
+excluded_locations_changed (GConfClient *client,
+			    guint cnxn_id,
+			    GConfEntry *entry,
+			    gpointer user_data)
 {
-	/* filesystem has changed (mounted or unmounted device) */
-	baobab_get_filesystem (&g_fs);
-	set_label_scan (&g_fs);
-	show_label ();
+	GSList *uris;
+
+	uris = 	gconf_client_get_list (client,
+				       BAOBAB_EXCLUDED_DIRS_KEY,
+				       GCONF_VALUE_STRING,
+				       NULL);
+	baobab_set_excluded_locations (uris);
+	g_slist_foreach (uris, (GFunc) g_free, NULL);
+	g_slist_free (uris);
+
+	baobab_update_filesystem ();
+
+	gtk_tree_store_clear (baobab.model);
+	first_row ();
 }
 
 static void
-home_contents_changed (GFileMonitor      *file_monitor,
-		       GFile             *child,
-		       GFile             *other_file,
-		       GFileMonitorEvent  event_type,
-		       gpointer           user_data)
-
+baobab_monitor_home_toggled (GConfClient *client,
+			     guint cnxn_id,
+			     GConfEntry *entry,
+			     gpointer user_data)
 {
-	gchar *excluding;
+	gboolean enable;
 
-	if (!baobab.bbEnableHomeMonitor)
-		return;
+	enable = gconf_client_get_bool (baobab.gconf_client,
+					BAOBAB_ENABLE_HOME_MONITOR_KEY,
+					NULL);
 
-	if (baobab.CONTENTS_CHANGED_DELAYED)
-		return;
-
-	excluding = g_file_get_basename (child);
-	if (strcmp (excluding, ".recently-used") == 0   ||
-	    strcmp (excluding, ".gnome2_private") == 0  ||
-	    strcmp (excluding, ".xsession-errors") == 0 ||
-	    strcmp (excluding, ".bash_history") == 0    ||
-	    strcmp (excluding, ".gconfd") == 0) {
-		g_free (excluding);
-		return;
-	}
-	g_free (excluding);
-
-	baobab.CONTENTS_CHANGED_DELAYED = TRUE;
-}
-
-static void
-monitor_volume (void)
-{
-	monitor_vol = g_volume_monitor_get ();
-	g_signal_connect (monitor_vol, "volume_changed",
-			  G_CALLBACK (volume_changed), NULL);
-}
-
-static void
-monitor_home_dir (void)
-{
-	GFile *file;
-	GError *error = NULL;
-	monitor_home = NULL;
-
-	file = g_file_new_for_path (g_get_home_dir ());
-	monitor_home = g_file_monitor_directory (file, 0, NULL, &error);
-	g_object_unref (file);
-
-	if (!monitor_home) {
-		message (_("Could not initialize monitoring"),
-			 _("Changes to your home folder will not be monitored."),
-			 GTK_MESSAGE_WARNING, NULL);
-		g_print ("homedir:%s\n", error->message);
-		g_error_free (error);
-	}
-	else {
-		g_signal_connect (monitor_home,
-				  "changed",
-				  G_CALLBACK (home_contents_changed),
-				  NULL);
-	}
+	monitor_home (enable);
 }
 
 static void
@@ -801,6 +865,10 @@ baobab_init (void)
 {
 	GSList *uri_list;
 	GError *error = NULL;
+	gboolean enable;
+
+	/* FileSystem usage */
+	baobab_get_filesystem (&baobab.fs);
 
 	/* Load the UI */
 	baobab.main_ui = gtk_builder_new ();
@@ -816,7 +884,6 @@ baobab_init (void)
 	gtk_builder_connect_signals (baobab.main_ui, NULL);
 
 	/* Misc */
-	baobab.label_scan = NULL;
 	baobab.CONTENTS_CHANGED_DELAYED = FALSE;
 	baobab.STOP_SCANNING = TRUE;
 	baobab.show_allocated = TRUE;
@@ -826,15 +893,17 @@ baobab_init (void)
 	baobab.gconf_client = gconf_client_get_default ();
 	gconf_client_add_dir (baobab.gconf_client, BAOBAB_KEY_DIR,
 			      GCONF_CLIENT_PRELOAD_NONE, NULL);
-	gconf_client_notify_add (baobab.gconf_client, PROPS_SCAN_KEY, props_notify,
+	gconf_client_notify_add (baobab.gconf_client, BAOBAB_EXCLUDED_DIRS_KEY, excluded_locations_changed,
+				 NULL, NULL, NULL);
+	gconf_client_notify_add (baobab.gconf_client, SYSTEM_TOOLBAR_STYLE_KEY, baobab_toolbar_style,
 				 NULL, NULL, NULL);
-	gconf_client_notify_add (baobab.gconf_client, SYSTEM_TOOLBAR_STYLE, baobab_toolbar_style,
-				 NULL, NULL, NULL);				 
 	gconf_client_notify_add (baobab.gconf_client, BAOBAB_SUBFLSTIPS_VISIBLE_KEY, baobab_subfolderstips_toggled,
 				 NULL, NULL, NULL);
+	gconf_client_notify_add (baobab.gconf_client, BAOBAB_ENABLE_HOME_MONITOR_KEY, baobab_monitor_home_toggled,
+				 NULL, NULL, NULL);
 
 	uri_list = gconf_client_get_list (baobab.gconf_client,
-						      PROPS_SCAN_KEY,
+						      BAOBAB_EXCLUDED_DIRS_KEY,
 						      GCONF_VALUE_STRING,
 						      NULL);
 
@@ -851,60 +920,63 @@ baobab_init (void)
 
 	monitor_volume ();
 
-	baobab.bbEnableHomeMonitor = gconf_client_get_bool (baobab.gconf_client,
-							    PROPS_ENABLE_HOME_MONITOR,
-							    NULL);
+	enable = gconf_client_get_bool (baobab.gconf_client,
+					BAOBAB_ENABLE_HOME_MONITOR_KEY,
+					NULL);
 
-	monitor_home_dir ();
+	monitor_home (enable);
 }
 
 static void
 baobab_shutdown (void)
 {
-	g_free (baobab.label_scan);
+	if (baobab.current_location)
+		g_object_unref (baobab.current_location);
 
-	if (current_location)
-		g_object_unref (current_location);
+	if (baobab.monitor_vol) {
+		g_object_unref (baobab.monitor_vol);
+	}
 
-	if (monitor_vol)
-		g_object_unref (monitor_vol);
-	if (monitor_home)
-		g_file_monitor_cancel (monitor_home);
+	if (baobab.monitor_home) {
+		g_file_monitor_cancel (baobab.monitor_home);
+		g_object_unref (baobab.monitor_home);
+	}
 
 	g_free (baobab.selected_path);
 
 	g_slist_foreach (baobab.excluded_locations, (GFunc) g_object_unref, NULL);
 	g_slist_free (baobab.excluded_locations);
 
-	if (baobab.gconf_client)
+	if (baobab.gconf_client) {
 		g_object_unref (baobab.gconf_client);
+	}
 }
 
-static void
+static BaobabChartMenu *
 create_context_menu (void)
 {
-	ContextMenu *menu = NULL;
+	BaobabChartMenu *menu = NULL;
 
-	baobab.chart_menu = g_new0 (ContextMenu, 1);
+	baobab.chart_menu = g_new0 (BaobabChartMenu, 1);
 	menu = baobab.chart_menu;
 
 	menu->widget = gtk_menu_new ();
-		
-	menu->up_item = gtk_image_menu_item_new_with_label (_("Move to parent folder")); 
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->up_item), 
+
+	menu->up_item = gtk_image_menu_item_new_with_label (_("Move to parent folder"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->up_item),
 				       gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
-		
+
 	menu->zoom_in_item = gtk_image_menu_item_new_with_label (_("Zoom in")) ;
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->zoom_in_item), 
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->zoom_in_item),
 				       gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
-        
+
 	menu->zoom_out_item = gtk_image_menu_item_new_with_label (_("Zoom out"));
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->zoom_out_item), 
-				       gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));       
-		
-	menu->snapshot_item = gtk_image_menu_item_new_with_label (_("Save snapshot"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->zoom_out_item),
+				       gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
+
+	menu->snapshot_item = gtk_image_menu_item_new_with_label (_("Save screenshot"));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu->snapshot_item),
-									gtk_image_new_from_file (BAOBAB_PIX_DIR "shot.png"));
+				       gtk_image_new_from_file (BAOBAB_PIX_DIR "shot.png"));
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu->widget),
 			       menu->up_item);
@@ -930,8 +1002,59 @@ create_context_menu (void)
 					  G_CALLBACK (on_chart_snapshot_cb), NULL);
 
 	gtk_widget_show_all (menu->widget);
+
+	return menu;
 }
 
+static void
+on_chart_item_activated (BaobabChart *chart, GtkTreeIter *iter)
+{
+	GtkTreePath *path;
+
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (baobab.model), iter);
+
+	if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (baobab.tree_view), path))
+		gtk_tree_view_expand_to_path (GTK_TREE_VIEW (baobab.tree_view), path);
+
+	gtk_tree_view_set_cursor (GTK_TREE_VIEW (baobab.tree_view),
+				  path, NULL, FALSE);
+	gtk_tree_path_free (path);
+}
+
+static gboolean
+on_chart_button_release (BaobabChart *chart,
+			 GdkEventButton *event,
+			 gpointer data)
+{
+	if (baobab_chart_is_frozen (baobab.current_chart))
+		return FALSE;
+
+	if (event->button== 3) /* right button */
+	{
+		GtkTreePath *root_path;
+		BaobabChartMenu *menu;
+
+		root_path = baobab_chart_get_root (baobab.current_chart);
+
+		menu = baobab.chart_menu;
+		gtk_widget_set_sensitive (menu->up_item,
+					  ((root_path != NULL) &&
+					  (gtk_tree_path_get_depth (root_path) > 1)));
+		gtk_widget_set_sensitive (menu->zoom_in_item,
+					  baobab_chart_can_zoom_in (baobab.current_chart));
+		gtk_widget_set_sensitive (menu->zoom_out_item,
+					  baobab_chart_can_zoom_out (baobab.current_chart));
+
+		/* show the menu */
+		gtk_menu_popup (GTK_MENU (menu->widget),
+				NULL, NULL, NULL, NULL,
+				event->button, event->time);
+
+		gtk_tree_path_free (root_path);
+	}
+
+	return FALSE;
+}
 
 static void
 drag_data_received_handl (GtkWidget *widget,
@@ -944,20 +1067,20 @@ drag_data_received_handl (GtkWidget *widget,
 			 gpointer data)
 {
 	GFile *gf = NULL;
-	
+
 	/* set "gf" if we got some valid data */
 	if ((selection_data != NULL) &&
-	    (selection_data->length >= 0) &&
+	    (gtk_selection_data_get_length (selection_data) >= 0) &&
 	    (target_type == DND_TARGET_URI_LIST)) {
 		gchar **uri_list;
-		uri_list = g_uri_list_extract_uris ((gchar*)selection_data->data);
+		uri_list = g_uri_list_extract_uris ((const gchar *) gtk_selection_data_get_data (selection_data));
 		/* check list is 1 item long */
 		if (uri_list != NULL && uri_list[0] != NULL && uri_list[1] == NULL) {
 			gf = g_file_new_for_uri (uri_list[0]);
 		}
 		g_strfreev (uri_list);
 	}
-	
+
 	/* success if "gf" has been set */
 	if (gf != NULL) {
 		/* finish drop before beginning scan, as the drag-drop can
@@ -987,9 +1110,9 @@ initialize_charts (void)
 	gtk_paned_set_position (GTK_PANED (hpaned_main), 480);
 
 	baobab.chart_type_combo = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (baobab.chart_type_combo), 
+	gtk_combo_box_append_text (GTK_COMBO_BOX (baobab.chart_type_combo),
 				   _("View as Rings Chart"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (baobab.chart_type_combo), 
+	gtk_combo_box_append_text (GTK_COMBO_BOX (baobab.chart_type_combo),
 				   _("View as Treemap Chart"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (baobab.chart_type_combo), 0);
 	gtk_widget_show (baobab.chart_type_combo);
@@ -1006,6 +1129,8 @@ initialize_charts (void)
 				   TRUE,
 				   0, GTK_PACK_END);
 
+	baobab.chart_menu = create_context_menu ();
+
 	/* Baobab's Treemap Chart */
 	baobab.treemap_chart = baobab_treemap_new ();
 	baobab_chart_set_model_with_columns (baobab.treemap_chart,
@@ -1019,7 +1144,7 @@ initialize_charts (void)
 	baobab_chart_set_max_depth (baobab.treemap_chart, 1);
 	g_signal_connect (baobab.treemap_chart, "item_activated",
 					G_CALLBACK (on_chart_item_activated), NULL);
-	g_signal_connect (baobab.treemap_chart, "button-release-event", 
+	g_signal_connect (baobab.treemap_chart, "button-release-event",
 					G_CALLBACK (on_chart_button_release), NULL);
 	g_signal_connect (baobab.treemap_chart, "drag-data-received",
 					G_CALLBACK (drag_data_received_handl), NULL);
@@ -1043,7 +1168,7 @@ initialize_charts (void)
 	baobab_chart_set_max_depth (baobab.rings_chart, 1);
 	g_signal_connect (baobab.rings_chart, "item_activated",
 					G_CALLBACK (on_chart_item_activated), NULL);
-	g_signal_connect (baobab.rings_chart, "button-release-event", 
+	g_signal_connect (baobab.rings_chart, "button-release-event",
 					G_CALLBACK (on_chart_button_release), NULL);
 	g_signal_connect (baobab.rings_chart, "drag-data-received",
 					G_CALLBACK (drag_data_received_handl), NULL);
@@ -1059,15 +1184,12 @@ initialize_charts (void)
 			   baobab.current_chart);
 	gtk_widget_show_all (chart_frame);
 
-	create_context_menu ();
-
 	check_drop_targets (FALSE);
 }
 
 static gboolean
 start_proc_on_command_line (GFile *file)
 {
-		
 	baobab_scan_location (file);
 
 	return FALSE;
@@ -1130,25 +1252,23 @@ main (int argc, char *argv[])
 
 	baobab_init ();
 
-	g_noactivescans = TRUE;
-	check_menu_sens (FALSE);
-	baobab_get_filesystem (&g_fs);
-	if (g_fs.total == 0) {
-		GtkWidget *dialog = gtk_message_dialog_new (NULL,
-                                  GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  GTK_MESSAGE_ERROR,
-                                  GTK_BUTTONS_CLOSE,
-                                  _("Could not detect any mount point."));
-                gtk_message_dialog_format_secondary_text
-                                                        (GTK_MESSAGE_DIALOG(dialog),
-                                                         _("Without mount points disk usage cannot be analyzed."));
+	if (baobab.fs.total == 0) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_CLOSE,
+				_("Could not detect any mount point."));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG(dialog),
+				_("Without mount points disk usage cannot be analyzed."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		goto closing;
 	}
 
-	set_label_scan (&g_fs);
-	show_label ();
+	check_menu_sens (FALSE);
+	update_scan_label ();
 
 	baobab.window = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "baobab_window"));
 	gtk_window_set_position (GTK_WINDOW (baobab.window),
@@ -1166,7 +1286,7 @@ main (int argc, char *argv[])
 	gtk_widget_show (baobab.window);
 
 	first_row ();
-	set_statusbar (_("Ready"));
+	baobab_set_statusbar (_("Ready"));
 
 	/* The ringschart */
 	initialize_charts ();
