@@ -30,21 +30,13 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-authentication-manager.h>
-#include <libgnomeui/gnome-help.h>
-#include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprint/gnome-print-pango.h>
-#include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
 
 #include "gdict-applet.h"
 #include "gdict-about.h"
 #include "gdict-pref-dialog.h"
 #include "gdict-print.h"
 #include "gdict-common.h"
-
-#include "gtkalignedwindow.h"
+#include "gdict-aligned-window.h"
 
 #define GDICT_APPLET_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GDICT_TYPE_APPLET, GdictAppletClass))
 #define GDICT_APPLET_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GDICT_TYPE_APPLET, GdictAppletClass))
@@ -149,7 +141,7 @@ set_window_default_size (GdictApplet *applet)
   defbox = priv->defbox;
 
   /* Size based on the font size */
-  font_size = pango_font_description_get_size (defbox->style->font_desc);
+  font_size = pango_font_description_get_size (gtk_widget_get_style (defbox)->font_desc);
   font_size = PANGO_PIXELS (font_size);
 
   width = font_size * WINDOW_NUM_COLUMNS;
@@ -162,7 +154,8 @@ set_window_default_size (GdictApplet *applet)
 
   /* ... but make it no larger than half the monitor size */
   screen = gtk_widget_get_screen (widget);
-  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+  monitor_num = gdk_screen_get_monitor_at_window (screen,
+                                                  gtk_widget_get_window (widget));
 
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
@@ -197,7 +190,6 @@ print_cb (GtkWidget   *widget,
     return;
   
   gdict_show_print_dialog (GTK_WINDOW (priv->window),
-  			   _("Print"),
   			   GDICT_DEFBOX (priv->defbox));
 }
 
@@ -332,7 +324,7 @@ gdict_applet_build_window (GdictApplet *applet)
       return;
     }
   
-  window = gtk_aligned_window_new (priv->toggle);
+  window = gdict_aligned_window_new (priv->toggle);
   g_signal_connect (window, "key-press-event",
 		    G_CALLBACK (window_key_press_event_cb),
 		    applet);
@@ -357,8 +349,8 @@ gdict_applet_build_window (GdictApplet *applet)
   
   gtk_box_pack_start (GTK_BOX (vbox), priv->defbox, TRUE, TRUE, 0);
   gtk_widget_show (priv->defbox);
-  GTK_WIDGET_SET_FLAGS (priv->defbox, GTK_CAN_FOCUS);
-  GTK_WIDGET_SET_FLAGS (priv->defbox, GTK_CAN_DEFAULT);
+  gtk_widget_set_can_focus (priv->defbox, TRUE);
+  gtk_widget_set_can_default (priv->defbox, TRUE);
   
   bbox = gtk_hbutton_box_new ();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
@@ -430,6 +422,8 @@ gdict_applet_icon_toggled_cb (GtkWidget   *widget,
       
       priv->is_window_showing = FALSE;
     }
+
+  return FALSE;
 }
 
 static void
@@ -510,7 +504,6 @@ gdict_applet_draw (GdictApplet *applet)
   GdictAppletPrivate *priv = applet->priv;
   GtkWidget *box;
   GtkWidget *hbox;
-  GtkWidget *label;
   gchar *text = NULL;
 
   if (priv->entry)
@@ -597,7 +590,7 @@ gdict_applet_draw (GdictApplet *applet)
 		  	    _("Dictionary entry"),
 			    _("Look up words in dictionaries"));
   
-  gtk_entry_set_editable (GTK_ENTRY (priv->entry), TRUE);
+  gtk_editable_set_editable (GTK_EDITABLE (priv->entry), TRUE);
   gtk_entry_set_width_chars (GTK_ENTRY (priv->entry), 12);
   g_signal_connect (priv->entry, "activate",
   		    G_CALLBACK (gdict_applet_entry_activate_cb),
@@ -714,14 +707,6 @@ gdict_applet_cmd_print (BonoboUIComponent *component,
   print_cb (NULL, applet);
 }
 
-static void
-gdict_applet_cmd_save (BonoboUIComponent *component,
-		       GdictApplet       *applet,
-		       const gchar       *cname)
-{
-  save_cb (NULL, applet);
-}
-
 static void
 gdict_applet_cmd_preferences (BonoboUIComponent *component,
 			      GdictApplet       *applet,
@@ -746,13 +731,11 @@ gdict_applet_cmd_help (BonoboUIComponent *component,
 		       const gchar       *cname)
 {
   GError *err = NULL;
+
+  gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (applet)),
+		"ghelp:gnome-dictionary#gnome-dictionary-applet",
+		gtk_get_current_event_time (), &err);
   
-  gnome_help_display_desktop_on_screen (NULL,
-  					"gnome-dictionary",
-  					"gnome-dictionary",
-  					"gnome-dictionary-applet",
-  					gtk_widget_get_screen (GTK_WIDGET (applet)),
-  					&err);
   if (err)
     {
       gdict_show_error_dialog (NULL,
@@ -781,18 +764,20 @@ gdict_applet_change_orient (PanelApplet       *applet,
 {
   GdictAppletPrivate *priv = GDICT_APPLET (applet)->priv;
   guint new_size;
+  GtkAllocation allocation;
   
+  gtk_widget_get_allocation (GTK_WIDGET (applet), &allocation);
   switch (orient)
     {
     case PANEL_APPLET_ORIENT_LEFT:
     case PANEL_APPLET_ORIENT_RIGHT:
       priv->orient = GTK_ORIENTATION_VERTICAL;
-      new_size = GTK_WIDGET (applet)->allocation.width;
+      new_size = allocation.width;
       break;
     case PANEL_APPLET_ORIENT_UP:
     case PANEL_APPLET_ORIENT_DOWN:
       priv->orient = GTK_ORIENTATION_HORIZONTAL;
-      new_size = GTK_WIDGET (applet)->allocation.height;
+      new_size = allocation.height;
       break;
     }
   
@@ -982,18 +967,6 @@ gdict_applet_set_defbox_font (GdictApplet *applet,
 				priv->defbox_font);
 }
 
-static void
-gdict_applet_set_word (GdictApplet *applet,
-		       const gchar *word)
-{
-  GdictAppletPrivate *priv = applet->priv;
-  
-  if (priv->word)
-    g_free (priv->word);
-  
-  priv->word = g_strdup (word);
-}
-
 static void
 gdict_applet_set_context (GdictApplet  *applet,
 			  GdictContext *context)
@@ -1061,7 +1034,6 @@ gdict_applet_gconf_notify_cb (GConfClient *client,
 			      gpointer     user_data)
 {
   GdictApplet *applet = GDICT_APPLET (user_data);
-  GdictAppletPrivate *priv = applet->priv;
 
   if (strcmp (entry->key, GDICT_GCONF_PRINT_FONT_KEY) == 0)
     {
@@ -1293,8 +1265,9 @@ gdict_applet_factory (PanelApplet *applet,
 {
   gboolean retval = FALSE;
 
-  if ((!strcmp (iid, "OAFIID:GNOME_DictionaryApplet")) ||
-      (!strcmp (iid, "OAFIID:GNOME_GDictApplet")))
+  if (((!strcmp (iid, "OAFIID:GNOME_DictionaryApplet")) ||
+       (!strcmp (iid, "OAFIID:GNOME_GDictApplet"))) &&
+      gdict_create_data_dir ())
     {
       /* Set up the menu */
       panel_applet_setup_menu_from_file (applet, UIDATADIR,
@@ -1313,39 +1286,28 @@ gdict_applet_factory (PanelApplet *applet,
 
   return retval;
 }
-#endif /* !GDICT_APPLET_STAND_ALONE */
+
+/* this defines the main () for the applet */
+PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_DictionaryApplet_Factory",
+			     GDICT_TYPE_APPLET,
+			     "gnome-dictionary-applet",
+			     VERSION,
+			     gdict_applet_factory,
+			     NULL);
+
+#else /* GDICT_APPLET_STAND_ALONE */
 
 int
 main (int argc, char *argv[])
 {
-#ifdef GDICT_APPLET_STAND_ALONE
   GtkWidget *window;
   GtkWidget *applet;
-#endif
 
   /* gettext stuff */
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-#ifndef GDICT_APPLET_STAND_ALONE
-  gnome_authentication_manager_init();
-
-  gnome_program_init ("gnome-dictionary-applet", VERSION,
-		      LIBGNOMEUI_MODULE,
-		      argc, argv,
-		      GNOME_CLIENT_PARAM_SM_CONNECT, FALSE,
-		      GNOME_PROGRAM_STANDARD_PROPERTIES,
-		      NULL);
-
-  if (!gdict_create_data_dir ())
-    return EXIT_FAILURE;
-
-  return panel_applet_factory_main ("OAFIID:GNOME_DictionaryApplet_Factory",
-		                    GDICT_TYPE_APPLET,
-				    gdict_applet_factory,
-				    NULL);
-#else
   gtk_init (&argc, &argv);
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -1366,5 +1328,6 @@ main (int argc, char *argv[])
   gtk_main ();
   
   return 0;
-#endif /* !GDICT_APPLET_STAND_ALONE */
 }
+
+#endif /* !GDICT_APPLET_STAND_ALONE */
